'use strict';

var _type = require('graphql/type');

var _language = require('graphql/language');

var _comparators = require('./comparators');

var _ = require('../');

var _2 = _interopRequireDefault(_);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env jest */

const querySource = `
 type Query {
   ok: Boolean!
 }
`;
const generateQuery = resolve => new _type.GraphQLObjectType({
  name: 'Query',
  fields: { ok: { type: new _type.GraphQLNonNull(_type.GraphQLBoolean), resolve } }
});
const schemaSource = `
  schema {
    query: Query
  }
`;
const Schema = new _type.GraphQLSchema({ query: generateQuery() });
const timestampSource = `
  scalar Timestamp
`;
const serialize = date => Date.prototype.toISOString.call(date);
const Timestamp = new _type.GraphQLScalarType({ name: 'Timestamp', serialize });
const recordSource = `
  interface Record {
    id: ID!
    createdAt: Timestamp!
    updatedAt: Timestamp!
  }
`;
const Record = new _type.GraphQLInterfaceType({
  name: 'Record',
  fields: {
    id: { type: new _type.GraphQLNonNull(_type.GraphQLID) },
    createdAt: { type: new _type.GraphQLNonNull(Timestamp) },
    updatedAt: { type: new _type.GraphQLNonNull(Timestamp) }
  }
});

describe('buildTypes()', () => {
  it('should work without types in AST', () => {
    expect((0, _.buildTypes)((0, _language.parse)(schemaSource))).toEqual([]);
  });

  it('should work with types in AST', () => {
    (0, _.buildTypes)((0, _language.parse)(querySource)).forEach((type, i) => (0, _comparators.expectTypesEqual)(type, [generateQuery()][i]));
  });

  it('should work with config map', () => {
    const ok = () => true;
    (0, _.buildTypes)((0, _language.parse)(querySource), { Query: { ok } }).forEach((type, i) => (0, _comparators.expectTypesEqual)(type, [generateQuery(ok)][i]));
  });

  it('should work with type dependencies', () => {
    (0, _.buildTypes)((0, _language.parse)(recordSource), undefined, [Timestamp]).forEach((type, i) => (0, _comparators.expectTypesEqual)(type, [Record, Timestamp][i]));
  });
});

describe('build()', () => {
  it('should throw if source is not a string', () => {
    expect(() => (0, _2.default)()).toThrowError('Expected a string but got undefined');
  });

  it('should throw if config is not an object', () => {
    expect(() => (0, _2.default)(querySource, null)).toThrowError('Expected an object but got null');
  });

  it('should throw if types is not an array', () => {
    expect(() => (0, _2.default)(querySource, undefined, null)).toThrowError('Expected an array but got null');
  });

  it('should throw with duplicate types', () => {
    const msg = `Duplicate type "${ Record.name }"`;
    expect(() => (0, _2.default)(recordSource, undefined, [Record])).toThrowError(msg);
    expect(() => (0, _2.default)(recordSource + recordSource)).toThrowError(msg);
    expect(() => (0, _2.default)(querySource + recordSource, undefined, [Record])).toThrowError(msg);
  });

  it('should throw with duplicate schema declarations', () => {
    expect(() => (0, _2.default)(schemaSource + schemaSource)).toThrowError('Must provide only one schema definition.');
  });

  it('should work with one type in source', () => {
    (0, _comparators.expectTypesEqual)((0, _2.default)(recordSource, undefined, [Timestamp]), Record);
  });

  it('should work with schema in source', () => {
    (0, _comparators.expectSchemasEqual)((0, _2.default)(schemaSource + querySource), Schema);
  });

  it('should work with inferred schema', () => {
    (0, _comparators.expectSchemasEqual)((0, _2.default)(querySource), Schema);
  });

  it('should work with multiple types in source', () => {
    Object.values((0, _2.default)(recordSource + timestampSource, {
      Timestamp: { serialize }
    })).forEach((type, i) => (0, _comparators.expectTypesEqual)(type, [Record, Timestamp][i]));
  });
});