'use strict';

var _type = require('graphql/type');

var _language = require('graphql/language');

var _buildFieldConfigMap = require('../buildFieldConfigMap');

var _buildFieldConfigMap2 = _interopRequireDefault(_buildFieldConfigMap);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const generateTypeAST = ({
  name = 'Object',
  fields = { field: 'Int' }
} = {}) => (0, _language.parse)(`
  type ${ name } {
    ${ Object.entries(fields).map(entry => entry.join(': ')).join('\n') }
  }
`).definitions[0]; /* eslint-env jest */

describe('buildFieldConfigArgumentMap()', () => {
  const generateArgumentFieldAST = ({
    name = 'field',
    args = { argument: 'Int' }
  } = {}) => generateTypeAST({
    fields: {
      [`
        ${ name }(
          ${ Object.entries(args).map(entry => entry.join(': ')).join('\n') }
        )
      `]: 'Int'
    }
  }).fields[0];
  const generateFieldConfigMap = ({
    name = 'argument',
    type = _type.GraphQLInt,
    defaultValue,
    description
  } = {}) => ({ [name]: { type, defaultValue, description } });

  it('should work with minimal AST', () => {
    expect((0, _buildFieldConfigMap.buildFieldConfigArgumentMap)(generateArgumentFieldAST())).toEqual(generateFieldConfigMap());
  });

  it('should work with description in AST', () => {
    const description = 'A description.';
    expect((0, _buildFieldConfigMap.buildFieldConfigArgumentMap)(generateArgumentFieldAST({
      args: { [`# ${ description }\nargument`]: 'Int' }
    }))).toEqual(generateFieldConfigMap({ description }));
  });

  it('should work with default value in AST', () => {
    expect((0, _buildFieldConfigMap.buildFieldConfigArgumentMap)(generateArgumentFieldAST({
      args: { argument: 'Int = 0' }
    }))).toEqual(generateFieldConfigMap({ defaultValue: 0 }));
  });
});

describe('buildFieldConfigMap()', () => {
  const generateFieldConfigMap = ({
    name = 'field',
    type = _type.GraphQLInt,
    args,
    resolve,
    deprecationReason,
    description
  } = {}) => ({ [name]: { type, args, resolve, deprecationReason, description } });
  it('should work with minimal AST', () => {
    expect((0, _buildFieldConfigMap2.default)(generateTypeAST())).toEqual(generateFieldConfigMap());
  });

  it('should work with description in AST', () => {
    const description = 'A description.';
    expect((0, _buildFieldConfigMap2.default)(generateTypeAST({
      fields: { [`# ${ description }\nfield`]: 'Int' }
    }))).toEqual(generateFieldConfigMap({ description }));
  });

  it('should work with arguments in AST', () => {
    expect((0, _buildFieldConfigMap2.default)(generateTypeAST({
      fields: { 'field(argument: Int)': 'Int' }
    }))).toEqual(generateFieldConfigMap({ args: { argument: { type: _type.GraphQLInt } } }));
  });

  it('should work with arguments in AST and `args` in config map', () => {
    const args = { argument: { type: _type.GraphQLInt } };
    expect((0, _buildFieldConfigMap2.default)(generateTypeAST({
      fields: { 'field(argument: Int)': 'Int' }
    }), { field: args })).toEqual(generateFieldConfigMap({ args }));
  });

  it('should work with `resolve` in config map', () => {
    const resolve = () => {};
    expect((0, _buildFieldConfigMap2.default)(generateTypeAST(), { field: { resolve } })).toEqual(generateFieldConfigMap({ resolve }));
  });

  it('should work with `deprecationReason` in config map', () => {
    const deprecationReason = 'A reason.';
    expect((0, _buildFieldConfigMap2.default)(generateTypeAST(), { field: { deprecationReason } })).toEqual(generateFieldConfigMap({ deprecationReason }));
  });
});